# !/Library/Frameworks/Python.framework/Versions/3.3/bin/python3
# !/usr/bin/env python3

#Copyright 2013 Embedded Data Systems, LLC
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from socket import *
import tkinter as tk
from tkinter import *
from tkinter import messagebox
from threading import Thread
import json
import queue

def CreateMainWin(master):
    # Create GUI
    global outputDisp
    bgnd = '#E0E0E0'
    
    frm = tk.Frame(master, borderwidth=10, bg=bgnd)
    tk.Button(frm, text='Discover Devices', command=RequestDevices, background=bgnd, highlightbackground=bgnd).grid(row=0)
    tk.Button(frm, text='Help', command=Help, background=bgnd, highlightbackground=bgnd).grid(row=0, column=4)
    tk.Button(frm, text='Quit', command=lambda master=master:End(master), background=bgnd, highlightbackground=bgnd).grid(row=0, column=5)
    
    outputDisp = tk.Text(frm, height=30, background='white', highlightbackground='grey', highlightthickness=1)
    outputDisp.grid(row=8, column=0, columnspan=6, sticky=N+S+E+W)
    sb = tk.Scrollbar(frm)
    sb.grid(row=8, column=6, sticky=N+S)
    sb.config(command=outputDisp.yview)
    outputDisp.config(yscrollcommand=sb.set)
    frm.pack(fill=BOTH, expand=1)

    master.configure(background='#F0F0F0')
    master.title("EDS Scanner - v0.02")
    master.resizable(0,0)

def End(rt):
    # End the program
    rt.destroy()

def Help():
    # Display help message
    helpMsg = 'This program locates all EDS web appliances (eg. OW Server\'s, MeshNet Controller\'s) on a sub-net and \
displays the IP address along with other information. \
This is accomplished by sending a UDP broadcast message and listening on port 30303. \
See the readme.pdf file for more information'
    tk.messagebox.showinfo("Help", helpMsg)

def RequestDevices():
    # Send request message to all devices
    outputDisp.delete(1.0, END)
    WriteHeader()
    sock.sendto(bytes("D","utf-8"), ("255.255.255.255", 30303))

def WriteHeader():
    # Write the header to output window
    outputDisp.insert(END, '{:<32}'.format(' Name') + '{:^20}'.format('Address') + '{:^10}'.format('Version') + '\n')
    outputDisp.insert(END, '{:<32}'.format('------') + '{:^20}'.format('---------') + '{:^10}'.format('---------') + '\n')
    outputDisp.yview(END)
    
def SetupSocket():
    # Create the UDP broadcast socket
    cs = socket(AF_INET, SOCK_DGRAM)
    cs.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
    cs.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
    cs.bind(('0.0.0.0', 30303))
    return cs

def ReadSocket(s):
    # Continuously read the socket and write the data to output
    while True:
        rcv = s.recv(2048).decode('utf-8')
        if rcv.find('{') >= 0:
            rcv = rcv[rcv.find('{'):rcv.find('}')+1]
            # This code removes a comma after the last field, which is sent in some older versions of EDS products
            commaErr = rcv.find(',\r\n}')
            if commaErr > 64:
                rcv = rcv[:commaErr] + '}'
            j = json.loads(rcv)
            msgStr = '{:<32}'.format(j['Name']) + '{:^20}'.format(j['IP']) + '{:^10}'.format('v' + j['FWVer']) + '\n'
            wQueue.put(msgStr)

def ReadQueue():
    # Read the output queue in the GUI loop to avoid threading problems
    while wQueue.empty() == FALSE:
        os = wQueue.get()
        outputDisp.insert(END, os)
        outputDisp.yview(END)
    rt.after(500, ReadQueue)
    
def main():
    # Program starts here
    global sock,rt,wQueue

    wQueue = queue.Queue()
    sock = SetupSocket()
    rt = tk.Tk()
    CreateMainWin(rt)
    sockRxThread = Thread(target=ReadSocket, args=[sock])
    sockRxThread.start()
    WriteHeader()
    RequestDevices()
    rt.after(100,ReadQueue)
    rt.mainloop()   

    sockRxThread._stop()
    sys.exit(0)

if __name__ == "__main__":
    main()
